unit Template;

interface

uses
  Classes, CustomVisitor, DabBoozePlant;

type
  TReportTemplate = class(TObject)
  private
    FReportStrings : TStrings;
    FBoozePlants: TDabBoozePlants;

    // Basic algorithm is private and not virtual so descendants don't override it
    procedure Report;
  protected
    // Elements of the reporting algorithm that must be overridden
    function  GetVisitor : TCustomVisitor; virtual; abstract;

    // Elements of the reporting algorithm that can be overridden
    procedure AddHeader; virtual;
    procedure AddFooter(Visitor : TCustomVisitor); virtual;

    // Elements that need to be accessed by descendants
    property ReportStrings : TStrings read FReportStrings;
    property BoozePlants   : TDabBoozePlants read FBoozePlants;
  public
    class procedure Execute(AReportStrings : TStrings;
                            ABoozePlants : TDabBoozePlants);
  end;


  // Concrete reports
  TEmployeeListReport = class(TReportTemplate)
  protected
    function  GetVisitor : TCustomVisitor; override;
    procedure AddHeader; override;
  end;


  TEmployeePayReport = class(TReportTemplate)
  protected
    function  GetVisitor : TCustomVisitor; override;
    procedure AddHeader; override;
    procedure AddFooter(Visitor : TCustomVisitor); override;
  end;


implementation

uses
  Visitor, SysUtils;

{ TReportTemplate }

class procedure TReportTemplate.Execute(AReportStrings : TStrings;
                                        ABoozePlants : TDabBoozePlants);
begin
  with Create do begin
    try
      FReportStrings := AReportStrings;
      FBoozePlants   := ABoozePlants;
      Report;
    finally
      Free;
    end;
  end;
end;


procedure TReportTemplate.AddFooter;
begin

end;


procedure TReportTemplate.AddHeader;
begin

end;


procedure TReportTemplate.Report;
  var
    TempVisitor : TCustomVisitor;
begin
  TempVisitor := GetVisitor;

  try
    FReportStrings.Clear;
    AddHeader;
    // In our case the travesral is done by the Visitor pattern
    FBoozePlants.Accept(TempVisitor);
    AddFooter(TempVisitor);
  finally
    TempVisitor.Free;
  end;
end;


{ TEmployeeListReport }

function TEmployeeListReport.GetVisitor : TCustomVisitor;
begin
  Result := TListEmployees.Create(ReportStrings);
end;


procedure TEmployeeListReport.AddHeader;
begin
  ReportStrings.Add('Employee List Report');
  ReportStrings.Add('');
end;


{ TEmployeePayReport }

function TEmployeePayReport.GetVisitor : TCustomVisitor;
begin
  Result := TListPayAmounts.Create(ReportStrings);
end;


procedure TEmployeePayReport.AddHeader;
begin
  ReportStrings.Add('Employee Pay Report');
  ReportStrings.Add('');
end;


procedure TEmployeePayReport.AddFooter(Visitor : TCustomVisitor);
begin
  ReportStrings.Add('');
  ReportStrings.Add('------------------------------------------------');
  ReportStrings.Add(Format('Total %f',
                           [TListPayAmounts(Visitor).TotalPayAmount]));
  ReportStrings.Add('------------------------------------------------');
end;


end.
